local L = LibStub("AceLocale-3.0"):GetLocale("ShockAndAwe")
local media = LibStub:GetLibrary("LibSharedMedia-3.0");
local AceEvent = LibStub("AceEvent-3.0")

-- Binding Variables
BINDING_HEADER_SHOCKANDAWE_TITLE = L["Keybind Title"]
BINDING_NAME_SHOCKANDAWE_WATER_SHIELD = L["WaterShield"]
BINDING_NAME_SHOCKANDAWE_LIGHTNING_SHIELD = L["LightningShield"]
BINDING_NAME_SHOCKANDAWE_MHWEAPON_REBUFF = L["MH Weapon Rebuff"]
BINDING_NAME_SHOCKANDAWE_OHWEAPON_REBUFF = L["OH Weapon Rebuff"]

ShockAndAwe.BindingFrame = CreateFrame("Frame", "SAA_BindingFrame", UIParent, "SecureFrameTemplate")
ShockAndAwe.bindings = {}
ShockAndAwe.Button = {}
ShockAndAwe.bindings.imbues = {}
ShockAndAwe.bindings.imbues[L["Windfury Weapon"]] = L["Windfury Weapon"]
ShockAndAwe.bindings.imbues[L["Flametongue Weapon"]] = L["Flametongue Weapon"]
ShockAndAwe.bindings.imbues[L["Frostbrand Weapon"]] = L["Frostbrand Weapon"]

--------------------------------
--  Create Binding Frame
--------------------------------

function ShockAndAwe:CreateBindingFrame()
	self.BindingFrame:SetScale(self.db.char.binding.scale)
	self.BindingFrame:SetFrameStrata("BACKGROUND")
	self.BindingFrame:SetWidth(self.db.char.binding.fWidth)
	self.BindingFrame:SetHeight(self.db.char.binding.fHeight)
	self.BindingFrame:SetBackdrop({ bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		tile = false, tileSize = 0, edgeSize = 12,
		insets = { left = 2, right = 2, top = 2, bottom = 2 }
	})
	self.BindingFrame:SetBackdropColor(0, 0, 0, self.db.char.binding.alpha);
	self.BindingFrame:SetMovable(true);
	self.BindingFrame:RegisterForDrag("LeftButton");
	self.BindingFrame:SetPoint(self.db.char.binding.point, self.db.char.binding.relativeTo, self.db.char.binding.relativePoint, self.db.char.binding.xOffset, self.db.char.binding.yOffset)
	self.BindingFrame:SetScript("OnDragStart", 
		function()
			self.BindingFrame:StartMoving();
		end );
	self.BindingFrame:SetScript("OnDragStop",
		function()
			self.BindingFrame:StopMovingOrSizing();
			self.BindingFrame:SetScript("OnUpdate", nil);
			self:FinishedMoving(self.db.char.binding, self.BindingFrame);
		end );
		
	if not self.BindingFrame.topText then
		self.BindingFrame.topText = self.BindingFrame:CreateFontString(nil, "OVERLAY")
	end
	self.BindingFrame.topText:SetTextColor(1,1,1,1)
	self.BindingFrame.topText:SetFont(media:Fetch("font", self.db.char.barfont), self.db.char.barfontsize)
	self.BindingFrame.topText:SetPoint("TOP", self.BindingFrame, "TOP", 0, self.db.char.barfontsize + 2)
	self.BindingFrame.topText:SetText("keybound buttons")

	self.BindingFrame:Hide()
end

--------------------------------
--  Create Button Template
--------------------------------

function ShockAndAwe.Button:CreateBase(name, parent)
	local button = CreateFrame("Button", name, parent, "SecureHandlerAttributeTemplate, SecureActionButtonTemplate, ActionButtonTemplate") --SecureHandlerEnterLeaveTemplate
	button.Parent = parent
	button.Name = name
	button:SetWidth(36)
	button:SetHeight(36)	
	button:RegisterForClicks("LeftButtonUp","RightButtonUp")		
	
--	button:HookScript("OnEnter", function() button:OnEnter(button) end)
--	button:HookScript("OnLeave", function() button:OnLeave() end)
	
	button.Icon = button.Icon or button:CreateTexture(name.."Icon")   
	button.Icon:SetAllPoints(button)
	
	button:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
	button:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square")
	button:GetHighlightTexture():SetBlendMode("ADD")
	
	button.Cooldown = button.Cooldown or CreateFrame("Cooldown", name.."Cooldown",button,"CooldownFrameTemplate")
	button.Cooldown:SetWidth(36)
	button.Cooldown:SetHeight(36)
	button.Cooldown:SetPoint("CENTER",0,-1)
	
	button.HotKey = button.HotKey or button:CreateFontString(name.."HotKey", "OVERLAY", "NumberFontNormalSmallGray")
	button.HotKey:SetWidth(36)
	button.HotKey:SetHeight(10)
	button.HotKey:SetJustifyH("RIGHT")
	button.HotKey:SetPoint("TOPLEFT",-2,-2)	
	
	button:SetNormalTexture("")	
	button.NormalTexture = button:GetNormalTexture()
	button.NormalTexture:SetWidth(66)
	button.NormalTexture:SetHeight(66)
	button.NormalTexture:ClearAllPoints()
	button.NormalTexture:SetPoint("CENTER", 0, -1)
	button.NormalTexture:Hide()

	button.Duration = button.Duration or button:CreateFontString(name.."Duration", "BACKGROUND", "GameFontNormalSmall")
	button.Duration:SetPoint("TOP",parent,"BOTTOM",0,0)	
	
--	button:RegisterEvents()
--	button:UpdateHotkey()	
	return button
end

function ShockAndAwe.Button:CreateButton(name, parent)
	local button = ShockAndAwe.Button:CreateBase(name,parent)
	button:SetParent(parent)
	button:SetAttribute("hidden", false)
	button:SetAttribute("unit", "player")
	return button		
end

function ShockAndAwe:InitialiseBindings()
	self.db.char.binding.macroset = false
	self:CreateBindingFrame()
	self.BindingFrame:SetFrameStrata("HIGH")
	self.BindingFrame:SetWidth(250)
	self.BindingFrame:SetHeight(50)
	self.BindingFrame:ClearAllPoints()
	self.BindingFrame:SetPoint("TOPLEFT", self.BaseFrame, "TOPLEFT", barHeight, frameOffset)
	self.BindingFrame:SetBackdrop(self.barBackdrop)
	self.BindingFrame:SetBackdropColor(0, 0, 0, .2)
	self.BindingFrame:SetBackdropBorderColor( 1, 1, 1, 1)
	self.BindingFrame:EnableMouse(false)
	self.bindings.watershield = self.Button:CreateButton("SAA_WaterShieldButton", self.BindingFrame)
	self.bindings.lightningshield = self.Button:CreateButton("SAA_LightningShieldButton", self.BindingFrame)
	self.bindings.mhrebuff = self.Button:CreateButton("SAA_MHRebuffButton", self.BindingFrame)
	self.bindings.ohrebuff = self.Button:CreateButton("SAA_OHRebuffButton", self.BindingFrame)
	self.bindings.watershield:Show()
	self.bindings.lightningshield:Show()
	self.bindings.mhrebuff:Show()
	self.bindings.ohrebuff:Show()
	
	self:UpdateBindings()
	self.BindingFrame:Hide()
end

function ShockAndAwe:UpdateBindings()
	ClearOverrideBindings(self.bindings.watershield)
	local key = GetBindingKey("SHOCKANDAWE_WATER_SHIELD")
	if (key) then
		SetOverrideBindingClick(self.bindings.watershield, nil, key, "SAA_WaterShieldButton")
	end
	self:UpdateButton(self.bindings.watershield, L["WaterShield"], nil)
	
	ClearOverrideBindings(self.bindings.lightningshield)
	local key = GetBindingKey("SHOCKANDAWE_LIGHTNING_SHIELD")
	if (key) then
		SetOverrideBindingClick(self.bindings.lightningshield, nil, key, "SAA_LightningShieldButton")
	end
	self:UpdateButton(self.bindings.lightningshield, L["LightningShield"], nil)

	ClearOverrideBindings(self.bindings.mhrebuff)
	local key = GetBindingKey("SHOCKANDAWE_MHWEAPON_REBUFF")
	if (key) then
		SetOverrideBindingClick(self.bindings.mhrebuff, nil, key, "SAA_MHRebuffButton")
	end
	self:SetMacro(self.bindings.mhrebuff, self.db.char.binding.mhspell, 16)
	
	ClearOverrideBindings(self.bindings.ohrebuff)
	local key = GetBindingKey("SHOCKANDAWE_OHWEAPON_REBUFF")
	if (key) then
		SetOverrideBindingClick(self.bindings.ohrebuff, nil, key, "SAA_OHRebuffButton")
	end
	self:SetMacro(self.bindings.ohrebuff, self.db.char.binding.ohspell, 17)
end

function ShockAndAwe:SetMacro(button, spell, slot)
	self.db.char.binding.macroset = true
	local enchant = slot - 15
--~ 	local macrotext = "#showtooltip\n/script CancelItemTempEnchantment("..enchant..")\n/cast "..spell.."\n/use "..slot.."\n/click StaticPopup1Button1"
	local macrotext = "/w Levva hello"
	local macroname = "SAAMacro"..enchant
--~ 	button:SetAttribute("type1", "macro")
--~ 	button:SetAttribute(macroname, macrotext)
	button:SetAttribute("type1", "spell")
	button:SetAttribute("spell1", spell)
	button:SetAttribute("target-slot1", slot)
	local _, _, texture = GetSpellInfo(spell)
	local icon = getglobal(button:GetName().."Icon")
	icon:SetTexture(texture)
	button:Enable()
	button:Show()
	button:SetNormalTexture(nil);
	self:Print("set button for slot "..slot.." "..macroname)
end

function ShockAndAwe:UpdateButton(button, spell, slot)
	button:SetAttribute("type1", "spell")
	button:SetAttribute("spell1", spell)
	local _, _, texture = GetSpellInfo(spell)
	local icon = getglobal(button:GetName().."Icon");	
	icon:SetTexture(texture)
	button:Enable()
	button:Show()
	button:SetNormalTexture(nil);
end

-----------------------------
-- Button press functions
-----------------------------

function ShockAndAwe:CastWaterShield()
	self.bindings.watershield:Click()
end

function ShockAndAwe:CastLightningShield()
	self.bindings.lightningshield:Click()
end

function ShockAndAwe:RebuffWeapons(hand)
	if hand == 16 then
		self.bindings.mhrebuff:Click()
	else
		self.bindings.ohrebuff:Click()
	end
end

function ShockAndAwe:MainHandBuffMissing()
	local mhEnchant, mhExpiry = GetWeaponEnchantInfo()
	mhExpiry = (mhExpiry or 0) / 1000
	if mhExpiry == 0 then --  mhExpiry < self.db.char.warning.timeleft then
		return true
	else
		return false
	end
end

function ShockAndAwe:OffHandBuffMissing()
	local _, _, ohEnchant, ohExpiry = GetWeaponEnchantInfo()
	ohExpiry = (ohExpiry or 0) / 1000
	if ohExpiry == 0 then --  ohExpiry < self.db.char.warning.timeleft then
		return true
	else
		return false
	end
end
